@@ -118,9 +118,60 @@ def parse_url(url, options):
 		options.zoom = zoom
 	if lat and lon and not options.center:
 		options.center = [lon, lat]
-	if not options.size and not options.size_px and not options.a and not options.fit and not options.bbox:
+	if not options.size and not options.size_px and not options.paper and not options.fit and not options.bbox:
 		options.size_px = [1280, 1024]
 
+def get_paper_size(name):
+	"""Returns paper size for name, [long, short] sides in mm"""
+	# ISO A*
+	m = re.match(r'^a?(\d)$', name)
+	if m:
+		return [math.floor(1000 / 2**((2*int(m.group(1)) - 1) / 4.0) + 0.2), math.floor(1000 / 2**((2*(int(m.group(1)) + 1) - 1) / 4.0) + 0.2)]
+	# ISO B*
+	m = re.match(r'^b(\d)$', name)
+	if m:
+		return [math.floor(1000 / 2**((int(m.group(1)) - 1) / 2.0) + 0.2), math.floor(1000 / 2**(int(m.group(1)) / 2.0) + 0.2)]
+	# German extensions
+	if name == '2a0':
+		return [2378, 1682]
+	if name == '4a0':
+		return [1682, 1189]
+	# US Legal
+	if re.match(r'^leg', name):
+		return [355.6, 215.9]
+	# US Letter
+	if re.match(r'^l', name):
+		return [279.4, 215.9]
+	# Cards
+	if re.match(r'^c(?:re|ar)d', name):
+		return [85.6, 54]
+	return None
+
+def xml_vars(style, variables):
+	"""Replace ${name:default} from style with variables[name] or 'default'"""
+	# Convert variables to a dict
+	v = {}
+	for kv in variables:
+		keyvalue = kv.split('=', 1)
+		if len(keyvalue) > 1:
+			v[keyvalue[0]] = keyvalue[1].replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;').replace('"', '&quot;').replace("'", '&#39;')
+	# Scan all variables in style
+	r = re.compile(r'\$\{([a-z0-9]+)(?::([^}]*))?\}')
+	rstyle = ''
+	last = 0
+	for m in r.finditer(style):
+		if m.group(1) in v:
+			value = v[m.group(1)]
+		elif m.group(2) is not None:
+			value = m.group(2)
+		else:
+			raise Exception('Found required style parameter: ' + m.group(1))
+		rstyle = rstyle + style[last:m.start()] + value
+		last = m.end()
+	if last < len(style):
+		rstyle = rstyle + style[last:]
+	return rstyle
+
 if __name__ == "__main__":
 	parser = argparse.ArgumentParser(description='Tile-aware mapnik image renderer')
 	parser.add_argument('-z', '--zoom', type=float, help='Target zoom level')
@@ -128,7 +179,7 @@ if __name__ == "__main__":
 	parser.add_argument('--factor', type=float, help='Scale factor (affects ppi, default=1)', default=1)
 	parser.add_argument('-s', '--scale', type=float, help='Scale as in 1:100000 (specifying ppi is recommended)')
 	parser.add_argument('-b', '--bbox', nargs=4, type=float, metavar=('Xmin', 'Ymin', 'Xmax', 'Ymax'), help='Bounding box')
-	parser.add_argument('-a', type=int, choices=range(-7, 8), help='Paper format: -a 4 for landscape A4, -a -4 for portrait A4')
+	parser.add_argument('-a', '--paper', help='Paper format: -a +4 for landscape A4, -a -4 for portrait A4, -a letter for autorotated US Letter')
 	parser.add_argument('-d', '--size', nargs=2, metavar=('W', 'H'), type=int, help='Target dimensions in mm')
 	parser.add_argument('-x', '--size-px', nargs=2, metavar=('W', 'H'), type=int, help='Target dimensions in pixels')
 	parser.add_argument('-m', '--margin', type=int, help='Amount in mm to reduce paper size')
@@ -146,6 +197,8 @@ if __name__ == "__main__":
 	parser.add_argument('-t', '--tiles', type=int, choices=range(1, 13), default=1, help='Write N×N tiles, then join using imagemagick')
 	parser.add_argument('-v', '--debug', action='store_true', default=False, help='Display calculated values')
 	parser.add_argument('-f', '--format', dest='fmt', help='Target file format (by default looks at extension)')
+	parser.add_argument('--base', help='Base path for style file, in case it\'s piped to stdin')
+	parser.add_argument('--vars', nargs='*', help='List of variables (name=value) to substitute in style file (use ${name:default})')
 	parser.add_argument('style', help='Style file for mapnik')
 	parser.add_argument('output', help='Resulting image file')
 	options = parser.parse_args()
@@ -154,6 +207,7 @@ if __name__ == "__main__":
 	scale = None
 	size = None
 	bbox = None
+	rotate = False
 
 	if options.url:
 		parse_url(options.url, options)
@@ -169,9 +223,19 @@ if __name__ == "__main__":
 	need_cairo = fmt in ['svg', 'pdf']
 	
 	# get image size in millimeters
-	if options.a:
-		dim_mm = [math.floor(1000 / 2**((2*abs(options.a) - 1) / 4.0) + 0.2), math.floor(1000 / 2**((2*(abs(options.a) + 1.0) - 1) / 4) + 0.2)]
-		if options.a < 0:
+	if options.paper:
+		portrait = False
+		if options.paper[0] == '-':
+			portrait = True
+			options.paper = options.paper[1:]
+		elif options.paper[0] == '+':
+			options.paper = options.paper[1:]
+		else:
+			rotate = True
+		dim_mm = get_paper_size(options.paper.lower())
+		if not dim_mm:
+			raise Exception('Incorrect paper format: ' + options.paper)
+		if portrait:
 			dim_mm = [dim_mm[1], dim_mm[0]]
 	elif options.size:
 		dim_mm = options.size
@@ -218,6 +282,8 @@ if __name__ == "__main__":
 	# all calculations are in EPSG:3857 projection (it's easier)
 	if bbox:
 		bbox = transform.forward(mapnik.Box2d(*bbox))
+		if size and size[0] > size[1] and rotate and bbox.maxy - bbox.miny > bbox.maxx - bbox.minx:
+			size = [size[1], size[0]]
 
 	# calculate bbox through center, zoom and target size
 	if not bbox and options.center and size and scale:
@@ -226,9 +292,22 @@ if __name__ == "__main__":
 		h = size[1] * scale / 2
 		bbox = mapnik.Box2d(center.x-w, center.y-h, center.x+w, center.y+h)
 
+	# reading style xml into memory for preprocessing
+	if options.style == '-':
+		style_xml = sys.stdin.read()
+		style_path = ''
+	else:
+		with open(options.style, 'r') as style_file:
+			style_xml = style_file.read()
+		style_path = os.path.dirname(options.style)
+	if options.base:
+		style_path = options.base
+	if 'vars' in options:
+		style_xml = xml_vars(style_xml, options.vars)
+
 	# for layer processing we need to create the Map object
 	m = mapnik.Map(100, 100) # temporary size, will be changed before output
-	mapnik.load_map(m, options.style)
+	mapnik.load_map_from_string(m, style_xml, False, style_path)
 	m.srs = p3857.params()
 
 	# get bbox from layer extents
@@ -256,7 +335,7 @@ if __name__ == "__main__":
 		else:
 			raise Exception('Image dimensions or scale were not specified in any way')
 
-	if need_cairo and options.tiles > 1:
+	if options.output == '-' or (need_cairo and options.tiles > 1):
 		options.tiles = 1
 	if max(size[0], size[1]) / options.tiles > 16384:
 		raise Exception('Image size exceeds mapnik limit ({} > {}), use --tiles'.format(max(size[0], size[1]) / options.tiles, 16384))
